@@ -544,6 +544,7 @@ def s_text(text, f_rgb=None, b_rgb=None, styles: tuple = ()) -> str:
     :param f_rgb: the RGB color code for the text color
     :param b_rgb: the RGB color code for the background color
     :param styles: the styles to be applied to the text. Options are 'bold', 'tilt', 'underscore', and 'cancel'
+        (must be a tuple, multiple styles can be applied)
     :return: str
     """
 
@@ -572,7 +573,7 @@ def s_text(text, f_rgb=None, b_rgb=None, styles: tuple = ()) -> str:
         background_rgb = [max(0, min(255, int(c))) for c in b_rgb[:3]]
         text = f'\033[48;2;{background_rgb[0]};{background_rgb[1]};{background_rgb[2]}m{text}'
 
-    return text
+    return f'{text}\033[0m'
 
 
 def sprint(text, f_rgb=None, b_rgb=None, styles: tuple = (), sep=' ', end='\n', file=None) -> None:
@@ -582,10 +583,31 @@ def sprint(text, f_rgb=None, b_rgb=None, styles: tuple = (), sep=' ', end='\n',
     :param f_rgb: the RGB color code for the text color
     :param b_rgb: the RGB color code for the background color
     :param styles: the styles to be applied to the text. Options are 'bold', 'tilt', 'underscore', and 'cancel'
+        (must be a tuple, multiple styles can be applied)
     :param sep: the separator to be used in the print function
     :param end: the end character to be used in the print function
     :param file: the file where the output will be written
     :return: None
     """
 
-    print(f'{s_text(text, f_rgb, b_rgb, styles)}\033[0m', sep=sep, end=end, file=file)
+    print(f'{s_text(text, f_rgb, b_rgb, styles)}', sep=sep, end=end, file=file)
+
+
+class TextReader:
+    def __init__(self, file_path: str, line_processor: typing.Callable[[str], typing.Any] = lambda _: _.strip()):
+        """
+        initializes the TextReader with a file path and a parsing function.
+        :param file_path: a string representing the path to the file to be read.
+        :param line_processor: a function to apply to each line of the file. the default function strips whitespace from each line.
+        """
+
+        self.file_path = file_path
+        self.line_processor = line_processor
+
+    def __iter__(self):
+        return self.line_gen()
+
+    def line_gen(self):
+        with open(self.file_path, 'r', encoding='utf-8') as file:
+            for line in file:
+                yield self.line_processor(line)
