@@ -9,6 +9,7 @@ import torch.nn.functional
 import typing
 import warnings
 import processingtools.functions
+import torch.utils.data
 
 
 class QueueTensor:
@@ -209,44 +210,54 @@ def torch_img_show(img):
     plt.show()
 
 
-def torch_img_denormalize(imgs):
+def torch_img_denormalize(image, mean: typing.Union[tuple, list, None] = None, std: typing.Union[tuple, list, None] = None):
     """
-    :param imgs: torch tensor
+    :param image: torch tensor
+    :param mean: mean for normalization
+    :param std: standard deviation for normalization
     :return: denormalized images
     """
+    if (mean is None and std is not None) or (mean is not None and std is None):
+        raise ValueError("Both mean and std must be provided together or both must be None")
 
-    for i in range(imgs.shape[0]):
-        imgs[i] = (imgs[i] - torch.min(imgs[i]))
-        imgs[i] = imgs[i] / torch.max(imgs[i]) * 255
+    if mean is not None and std is not None:
+        for i in range(image.shape[0]):
+            image = (image * std + mean) * 255
+    else:
+        for i in range(image.shape[0]):
+            image[i] = (image[i] - torch.min(image[i]))
+            image[i] = image[i] / torch.max(image[i]) * 255
 
-    return imgs
+    return image
 
 
-def torch_imgs_save(imgs, save_path: str = './'):
+def torch_imgs_save(image, save_path: str = './', mean: typing.Union[tuple, list, None] = None, std: typing.Union[tuple, list, None] = None):
     """
     Save images in png files
-    :param imgs: torch tensor
+    :param image: torch tensor
     :param save_path: save path
+    :param mean: mean for normalization
+    :param std: standard deviation for normalization
     :return: True if normal, otherwise False
     """
 
-    if type(imgs) != torch.Tensor:
+    if type(image) != torch.Tensor:
         print('\033[31mInput must be torch tensor images (Tensor shape must be (?, 3, ?, ?))\033[0m')
         return False
 
-    imgs = torch.unsqueeze(torch.unsqueeze(imgs, dim=0), dim=0) if len(imgs.shape) == 2 else imgs
-    imgs = torch.unsqueeze(imgs, dim=0) if len(imgs.shape) == 3 else imgs
+    image = torch.unsqueeze(torch.unsqueeze(image, dim=0), dim=0) if len(image.shape) == 2 else image
+    image = torch.unsqueeze(image, dim=0) if len(image.shape) == 3 else image
 
-    if imgs.shape[1] != 3 and imgs.shape[1] != 1:
+    if image.shape[1] != 3 and image.shape[1] != 1:
         print('\033[31mInput must be torch tensor images (Tensor shape must be (?, 3, ?, ?) if color scale or (?, 1, ?, ?) if gray scale)\033[0m')
         return False
 
-    imgs = imgs.clone()
-    imgs = torch_img_denormalize(imgs)
-    zeros = int(np.log10(imgs.shape[0])) + 1
+    image = image.clone()
+    image = torch_img_denormalize(image, mean, std)
+    zeros = int(np.log10(image.shape[0])) + 1
 
-    for i in range(imgs.shape[0]):
-        img = imgs[i].cpu().detach().clone()
+    for i in range(image.shape[0]):
+        img = image[i].cpu().detach().clone()
         cv2.imwrite(f'{save_path}{i:0{zeros}}.png', np.array(img.permute(1, 2, 0)).astype(int)[:, :, ::-1])
 
     return True
@@ -535,7 +546,7 @@ class AutoInputModel(torch.nn.Module):
     """
 
     @processingtools.functions.custom_warning_format
-    def __init__(self, model, size: typing.Union[tuple, list], mean: typing.Union[float, list, torch.Tensor, None] = None, std: typing.Union[float, list, torch.Tensor, None] = None, transformer=None):
+    def __init__(self, model, size: typing.Union[tuple, list, None] = None, mean: typing.Union[float, list, torch.Tensor, None] = None, std: typing.Union[float, list, torch.Tensor, None] = None, transformer=None):
         """
         initialize the AutoInputModel
         :param model: model to be used
@@ -559,11 +570,14 @@ class AutoInputModel(torch.nn.Module):
             self.transform = transformer
         else:
             self.transform = torchvision.transforms.Compose([
+                torchvision.transforms.ToPILImage(),
                 torchvision.transforms.Resize(size=size),
                 torchvision.transforms.ToTensor(),
                 torchvision.transforms.Normalize(mean, std),
             ])
 
+        self.device = None
+
     def image_read(self, path: str) -> torch.Tensor:
         """
         read and preprocess an image from the given path
@@ -572,46 +586,98 @@ class AutoInputModel(torch.nn.Module):
         """
 
         try:
-            image = torch.from_numpy(cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)).float().permute(2, 0, 1)[None]
-            return self.transform(image)
+            image = cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)
+            return torch.unsqueeze(self.transform(image), dim=0)
         except Exception as e:
             raise ValueError(f'Error reading image from path {path}: {e}')
 
-    def get_device(self, device=None) -> None:
+    def get_device(self) -> None:
         """
         get the device to run the model on
-        :param device: device to run the model on
         :return: device
         """
 
-        device = next(self.model.parameters()).device if device is None else device
-        print(f'run on {processingtools.functions.s_text(f"{device}", styles=("bold",))}')
+        self.device = next(self.model.parameters()).device
+        print(f'run on {processingtools.functions.s_text(f"{self.device}", styles=("bold",))}')
 
     @processingtools.functions.custom_warning_format
-    def forward(self, inputs: typing.Union[str, list], device=None) -> torch.Tensor:
+    def forward(self, inputs: typing.Union[str, list], batch_size: int = 1, num_workers: int = 0) -> typing.Union[torch.Tensor, dict]:
         """
         forward pass of the model
         :param inputs: string or list of strings representing image paths
-        :param device: device to run the model on
+        :param batch_size: batch size if input is path list
+        :param num_workers: workers num for dataloader if input is path list
         :return: model outputs
         """
 
         if self.model.training:
             warnings.warn('model is training mode now! (if you want to change eval mode, add model.eval())')
 
-        self.get_device(device)
+        self.get_device()
 
-        outputs = []
         if isinstance(inputs, list):
-            for i in inputs:
-                outputs.append(self.model(self.image_read(i)))
-            return torch.cat(outputs, dim=0)
+            outputs = []
+            out_dict = {}
+
+            dataset = AutoInputDataset(inputs, transformer=self.transform)
+            dataloader = torch.utils.data.DataLoader(dataset, batch_size=batch_size, shuffle=False, num_workers=num_workers)
+
+            for data in processingtools.ProgressBar(dataloader, total=len(dataloader), finish_mark=None):
+                image, paths = data
+                image = image.to(self.device)
+                output = self.model(image)
+
+                outputs.append(output)
+
+                for out, path in zip(output, paths):
+                    out_dict[path] = out
+
+            print('\rInference done.')
+            out_dict['total outputs'] = torch.cat(outputs, dim=0)
+
+            return out_dict
+
         elif isinstance(inputs, str):
             return self.model(self.image_read(inputs))
         else:
             raise TypeError('inputs must be a string or a list of strings')
 
 
+class AutoInputDataset(torch.utils.data.Dataset):
+    def __init__(self, image_list: list, size: typing.Union[tuple, list, None] = None, mean: typing.Union[float, list, torch.Tensor, None] = None, std: typing.Union[float, list, torch.Tensor, None] = None, transformer=None):
+        """
+        initialize the AutoInputDataset
+        :param image_list:
+        :param size: size to which images will be resized
+        :param mean: mean for normalization
+        :param std: standard deviation for normalization
+        :param transformer: custom transformer for image preprocessing
+        """
+
+        if transformer is not None and (mean is not None or std is not None):
+            warnings.warn('NormalizeModel uses transformer for normalizing not (mean, std).')
+
+        if transformer is not None:
+            self.transform = transformer
+        else:
+            self.transform = torchvision.transforms.Compose([
+                torchvision.transforms.ToPILImage(),
+                torchvision.transforms.Resize(size=size),
+                torchvision.transforms.ToTensor(),
+                torchvision.transforms.Normalize(mean, std),
+            ])
+
+        self.image_list = image_list
+
+    def __len__(self):
+        return len(self.image_list)
+
+    def __getitem__(self, idx):
+        path = self.image_list[idx]
+        image = cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)
+        return self.transform(image), path
+
+
 def gpu_env() -> None:
     """
     check and display GPU availability and details.
